<?php

namespace Shpopping\Product\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Shpopping\Product\Events\ProductAction as ProductActionEvent;
use Shpopping\Product\Notifications\ProductAction as ProductActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ProductAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ProductActionEvent.
     *
     * @param   ProductActionEvent  $event
     * @return mixed
     */
    public function handle(ProductActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $product.
     *
     * @param   ProductActionEvent  $event
     * @return void
     */
    public function create(ProductActionEvent $event)
    {
        $client = $event->product->client;
        Notification::send($client, new ProductActionNotification($event));
    }

    /**
     * Handle the ProductActionEvent as a listener.
     *
     * @param   ProductActionEvent  $event
     * @return mixed
     */
    public function asListener(ProductActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ProductActionEvent  $event
     * @return bool
     */
    private function isAllowed(ProductActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
